@@ -1,18 +1,18 @@
-/*  Clzip - LZMA lossless data compressor
-    Copyright (C) 2010-2019 Antonio Diaz Diaz.
+/* Clzip - LZMA lossless data compressor
+   Copyright (C) 2010-2021 Antonio Diaz Diaz.
 
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
+   This program is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 2 of the License, or
+   (at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #define _FILE_OFFSET_BITS 64
@@ -159,16 +159,15 @@ static bool LZd_verify_trailer( struct LZ_decoder * const d,
     {
     if( verbosity >= 4 ) show_header( d->dictionary_size );
     if( data_size == 0 || member_size == 0 )
-      fputs( "no data compressed.  ", stderr );
+      fputs( "no data compressed. ", stderr );
     else
-      fprintf( stderr, "%6.3f:1, %5.2f%% ratio, %5.2f%% saved.  ",
+      fprintf( stderr, "%6.3f:1, %5.2f%% ratio, %5.2f%% saved. ",
                (double)data_size / member_size,
                ( 100.0 * member_size ) / data_size,
                100.0 - ( ( 100.0 * member_size ) / data_size ) );
     if( verbosity >= 4 ) fprintf( stderr, "CRC %08X, ", td_crc );
     if( verbosity >= 3 )
-      fprintf( stderr, "decompressed %9llu, compressed %8llu.  ",
-               data_size, member_size );
+      fprintf( stderr, "%9llu out, %8llu in. ", data_size, member_size );
     }
   return true;
   }
