@@ -1,18 +1,18 @@
-/*  Clzip - LZMA lossless data compressor
-    Copyright (C) 2010-2019 Antonio Diaz Diaz.
+/* Clzip - LZMA lossless data compressor
+   Copyright (C) 2010-2021 Antonio Diaz Diaz.
 
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 2 of the License, or
-    (at your option) any later version.
+   This program is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 2 of the License, or
+   (at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #define _FILE_OFFSET_BITS 64
@@ -95,11 +95,10 @@ bool Mb_init( struct Matchfinder_base * const mb, const int before_size,
   if( Mb_read_block( mb ) && !mb->at_stream_end &&
       mb->buffer_size < buffer_size_limit )
     {
-    uint8_t * tmp;
-    mb->buffer_size = buffer_size_limit;
-    tmp = (uint8_t *)realloc( mb->buffer, mb->buffer_size );
+    uint8_t * const tmp = (uint8_t *)realloc( mb->buffer, buffer_size_limit );
     if( !tmp ) { free( mb->buffer ); return false; }
     mb->buffer = tmp;
+    mb->buffer_size = buffer_size_limit;
     Mb_read_block( mb );
     }
   if( mb->at_stream_end && mb->stream_pos < dict_size )
@@ -167,7 +166,7 @@ void Re_flush_data( struct Range_encoder * const renc )
   }
 
 
-     /* End Of Stream mark => (dis == 0xFFFFFFFFU, len == min_match_len) */
+/* End Of Stream marker => (dis == 0xFFFFFFFFU, len == min_match_len) */
 void LZeb_full_flush( struct LZ_encoder_base * const eb, const State state )
   {
   int i;
